#include "resourcemanager.h"
#include <alleggl.h>
#include <stdexcept>

ResourceManager ::ResourceManager() : mainfont(0)
{
	textureIds.resize(MaxTexture + 1);
}

static int loadFromFile(const std::string & filename, bool alpha = false)
{
	PALETTE pal;
	BITMAP *bmp = load_bitmap(filename.c_str(), pal);
	if (bmp == 0) {
		throw std::runtime_error("Failed to load texture: " + filename);
	}
	int textureId = allegro_gl_make_texture_ex(AGL_TEXTURE_FLIP  /* flags */,
		bmp,
		alpha ? GL_RGBA : GL_RGB); // internal_format	
	destroy_bitmap(bmp);
	if (textureId == 0) {
		throw std::runtime_error("Failed to convert texture: " + filename);
	}
	return textureId;	
}

static SAMPLE * loadSample(const std::string & filename)
{
	SAMPLE *s = load_sample(filename.c_str());
	if (s == 0) {
		throw std::runtime_error("Failed to load sample: " + filename);
	}
	return s;
}

void ResourceManager::loadAll()
{
	// Don't convert these as they may contain alpha.
	set_color_conversion(COLORCONV_NONE);
	textureIds[BackgroundTexture] = loadFromFile("gfx/background.tga");
	textureIds[StarTexture] = loadFromFile("gfx/star.tga");
	textureIds[PointerTexture] = loadFromFile("gfx/pointer.tga", true);
	textureIds[StarSurfaceTexture] = loadFromFile("gfx/flametex1.tga");
	textureIds[GlowTexture] = loadFromFile("gfx/glow.tga", true);
	textureIds[ShipTexture] = loadFromFile("gfx/shiptex.tga");
	textureIds[TractorTexture] = loadFromFile("gfx/tractortexture.tga");
	
	models.resize(MaxModel);
	models[Ship].load("models/ship.model");
	
	samples.resize(MaxSample);
	// e.g. samples[Death] = loadSample("sounds/aargh.wav");
	mainfont = load_font("font.pcx", NULL, NULL);
	if (mainfont == 0) {
		throw std::runtime_error("Font load failed");
	}
}

ResourceManager resourceManager;

