#ifndef VECTOR3D_INCLUDED
 #define VECTOR3D_INCLUDED
#include <cmath>

// Quickly hacked from previous work; not pretty.

typedef double Component;

inline Component toRadians(Component deg)
{
	return (deg / 180) * M_PI;
}

inline Component toDegrees(Component rad)
{
	return (rad / M_PI) * 180;
}

class Vect3d  
{
public:
	Component x;
	Component y;
	Component z;
public:
	void normalise(void) { scalarprod(1 / magnitude()); }
	void scalarprod(Component n) { x *= n; y *= n; z *= n; }
	Component magnitude() const { return std::sqrt(magnitudeSquared()); }
	Component magnitudeSquared() const { return (x*x) + (y*y) + (z*z); }
	Vect3d() : x(0), y(0), z(0) { }
	Vect3d(Component a, Component b, Component c = 0) : x(a), y(b), z(c) { }
	Vect3d operator + (const Vect3d &b) const {
		return Vect3d(x+b.x, y+b.y, z+b.z);
	}
	/* subtraction */
	Vect3d operator - (const Vect3d &b) const {
		return Vect3d(x - b.x, y - b.y, z - b.z);
	}
	/* Unary - */
	Vect3d operator - () const {
		return Vect3d(-x, -y, -z);
	}
	/* Vector cross product */
	Vect3d operator * (const Vect3d &b) const;
	/* scalar product */
    	Vect3d operator * (Component s) const {
		return Vect3d(x * s, y * s, z * s);
	}
	/* dot product */
	Component dotProduct(const Vect3d &b) const {
		return (x * b.x) + (y * b.y) + (z * b.z);
	}
	static Vect3d fromPolar(Component angle, Component magnitude)
		{
			Component angleRad = toRadians(angle);
			// Work out the direction
			// 0 == North therefore we need to make sure it goes clockwise too...
			return Vect3d(
				std::sin(angleRad) * magnitude,
				std::cos(angleRad) * magnitude,
				0);
		}
	
	// Returns in degrees
	Component angle()
	{
		Component angleRad = std::atan2(x, y);
		return toDegrees(angleRad);
	}
};

inline void normaliseAngle(Component &a)
{
	if (a < -180) {
		a += 360;
	}
	if (a > 180) {
		a -= 360;
	}
}


#endif // VECTOR3D_INCLUDED
